package trailwalker.track.data;

import org.json.JSONException;
import org.json.JSONObject;

import android.location.Location;

/**
 * Classe responsable de contenir la informaci d'una posici. s utilitzada per altres classes
 * @author Albert Jord Vinardell
 * @version Desembre 2012
 * */
public class LocationInformation implements Comparable<LocationInformation> {
	//*******************************************************************************
    //Constants
    //*******************************************************************************
	private static final String KIND_OF_DEVICE = "Android";
	
	//*******************************************************************************
    //Variables
    //*******************************************************************************
	private Location _location;
	private String _wkid;
	private String _IMSI;
	private String _team;
	private int _batteryLevel;
	
	private double _x=0;
	private double _y=0;

	//*******************************************************************************
    //Constructor
    //*******************************************************************************
	/**
	 * Constructor
	 * @param location instncia de {@link Location} que cont les dades geogrfiques i de temps de la posici capturada
	 * @param wkid parmetre utilitzat en el missatge HTTP pel servidor GIS
	 * @param IMSI parmetre utilitzat en el missatge HTTP pel servidor GIS
	 * @param team parmetre per identificar l'equip
	 * @param batteryLevel nivell de bateria
	 * */
	public LocationInformation(Location location, String wkid, String IMSI, String team, int batteryLevel){
		this._location=location;
		this._wkid=wkid;
		this._IMSI=IMSI;
		this._team=team;
		this._batteryLevel=batteryLevel;
		this.ToWebMercator(location.getLongitude(), location.getLatitude());
	}
	
	/**
	 * Funci que torna les dades contingudes en la classe en una instncia de {@link JSONObject}
	 * @return {@link JSONObject}
	 * 
	 * */
	public JSONObject toJSONObject(){
		JSONObject root = new JSONObject();
		JSONObject geometry = new JSONObject();
		JSONObject attributes = new JSONObject();
		try {
			geometry.put("spatialReference",new JSONObject().put("wkid", this._wkid));
			geometry.put("y", _y);
			geometry.put("x", _x);

			attributes.put("CodiEquip", this._IMSI);
			attributes.put("Equip",this._team);
			attributes.put("Bateria",this._batteryLevel);
				
			attributes.put("Entrada",this._location.getTime());
			attributes.put("Tipus", LocationInformation.KIND_OF_DEVICE);
			
			root.put("attributes", attributes);
			root.put("geometry", geometry);
			
		} catch (JSONException e) {
			e.printStackTrace();
		}
		
		return root;
	}
	
	/**
	 * Funci que torna els milisegons des del 1970 de la captura de la posici
	 * @return Long amb el nombre de milisegons
	 * */
	public Long getTime(){
		return this._location.getTime();
	}
	
	@Override
	public int compareTo(LocationInformation another) {
		if(this.getTime()<another.getTime())return -1;
		else if(this.getTime()==another.getTime())return 0;
		else return 1;
	}
	
	/**
	 * Funci per convertir la latitut i longitut contingudes en les variables privades a format WebMercator
	 * */
	private void ToWebMercator(double mercatorX_lon,double mercatorY_lat)
	{
	    if ((Math.abs(mercatorX_lon) > 180 || Math.abs(mercatorY_lat) > 90))
	        return;

	    double num = mercatorX_lon * 0.017453292519943295;
	    double x = 6378137.0 * num;
	    double a = mercatorY_lat * 0.017453292519943295;

	    _x = x;
	    _y = 3189068.5 * Math.log((1.0 + Math.sin(a)) / (1.0 - Math.sin(a)));
	}

}
