package trailwalker.track.device;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.ContextWrapper;
import android.content.Intent;
import android.content.IntentFilter;
import android.util.Log;

/**
 * Classe que obt informaci relativa a l'estat de la bateria del dispositiu. Hereda de {@link BroadcastReceiver}
 * @author Albert Jord Vinardell
 * @version Desembre 2012
 * */
public class BatteryInformation extends BroadcastReceiver {
    //*******************************************************************************
    //Variables
    //*******************************************************************************
	private static int batteryLevelPercentage=-1;
	ContextWrapper cw = null;
	
    //*******************************************************************************
    //Constructor
    //*******************************************************************************
	/**
	 * Constructor
	 * @param context instncia de {@link Context} de l'aplicaci
	 * */
	public BatteryInformation (Context context){
		Log.d("TrailWalkerTrack","TrailWalkerTrack: BatteryInformation: BatteryInformation()");
		IntentFilter batteryLevelFilter = new IntentFilter(Intent.ACTION_BATTERY_CHANGED);
        cw = new ContextWrapper(context);
		//Subscripci per rebre els events de sistema del tipus ACTION_BATTERY_CHANGED
        cw.registerReceiver(this, batteryLevelFilter);
	}

    //*******************************************************************************
    //Funcions
    //*******************************************************************************
	/**
	 * Guarda en una variable global el percentatge de bateria del dispositiu.
	 * Sobreescriptura de la funci "onReceive" de BroadcastReceiver. Es reben els events de sistema relatius a la subscripci ACTION_BATTERY_CHANGED.
	 * @param context instncia de {@link Context} de l'aplicaci
	 * @param intent instncia de {@link Intent} relacionada amb l'event
	 * */
	@Override
    public void onReceive(Context context, Intent intent) {
		Log.d("TrailWalkerTrack","TrailWalkerTrack: BatteryInformation: onReceive()");
        int rawlevel = intent.getIntExtra("level", -1);
        int scale = intent.getIntExtra("scale", -1);
        BatteryInformation.batteryLevelPercentage = -1;
        if (rawlevel >= 0 && scale > 0) {
        	batteryLevelPercentage = (rawlevel * 100) / scale;
        }
    }
	
	/**
	 * Torna el percentatge de nivell de bateria del dispositiu
	 * @return enter amb el nivell de bateria
	 * */
	public int getBatteryLevelPercentage(){
		Log.d("TrailWalkerTrack","TrailWalkerTrack: BatteryInformation: getBatteryLevelPercentage()");
		return BatteryInformation.batteryLevelPercentage;
	}
	
	/**
	 * Destructor de la classe
	 * */
	public void finalize() throws Throwable{
		Log.d("TrailWalkerTrack","TrailWalkerTrack: BatteryInformation: finalize()");
		// Finalitzaci de la subscripci d'events de sistema
		cw.unregisterReceiver(this);
		super.finalize();
	} 
	
}
