package trailwalker.track.device;

import android.content.Context;
import android.telephony.TelephonyManager;
import android.util.Log;

/**
 * Classe que obt informaci diversa del dispositiu
 * @author Albert Jord Vinardell
 * @version Desembre 2012
 * */
public class DeviceInformationRetriever {
	
    //*******************************************************************************
    //Variables
    //*******************************************************************************
	private Context mContext;
	private BatteryInformation bi = null;
	private String iMEI = null;
	private String iMSI = null;
	
    //*******************************************************************************
    //Constructor
    //*******************************************************************************
	/**
	 * Constructor
	 * @param context instncia de {@link Context} de l'aplicaci
	 * */
	public DeviceInformationRetriever(Context context){
		Log.d("TrailWalkerTrack","TrailWalkerTrack: DeviceInformationRetriever: DeviceInformationRetriever()");
		this.mContext = context;
        bi = new BatteryInformation(this.mContext);
	}

    //*******************************************************************************
    //Funcions
    //*******************************************************************************	
	/**
	 * Funci que torna l'IMEI del dispositiu
	 * @return IMEI del dispositiu
	 * */
	public String getIMEI() {
		Log.d("TrailWalkerTrack","TrailWalkerTrack: DeviceInformationRetriever: getIMEI()");
		if(iMEI==null){
			String serviceName = Context.TELEPHONY_SERVICE;
			TelephonyManager m_telephonyManager = (TelephonyManager)mContext.getSystemService(serviceName);
			iMEI = m_telephonyManager.getDeviceId();
		}
		return iMEI;
	}
	
	/**
	 * Funci que torna l'IMSI del dispositiu
	 * @retur IMSI del dispositiu
	 * */
	public String getIMSI(){
		Log.d("TrailWalkerTrack","TrailWalkerTrack: DeviceInformationRetriever: getIMSI()");
		if(iMSI==null){
			String serviceName = Context.TELEPHONY_SERVICE;
			TelephonyManager m_telephonyManager = (TelephonyManager)mContext.getSystemService(serviceName);
			iMSI = m_telephonyManager.getSubscriberId();
		}
		return iMSI;
	}
	
	/**
	 * Funci que torna un enter amb el percentatge de bateria del dispositiu
	 * @return percentatge de bateria del dispositiu
	 * */
	public int getBatteryLevelPercentage(){
		Log.d("TrailWalkerTrack","TrailWalkerTrack: DeviceInformationRetriever: getBatteryLevelPercentage()");
		return bi.getBatteryLevelPercentage();
	}
	
	/**
	 * Destructor de la classe.
	 * */
	public void finalize() throws Throwable{
		Log.d("TrailWalkerTrack","TrailWalkerTrack: DeviceInformationRetriever: finalize()");
		bi.finalize();
		super.finalize();
	} 

}
