package trailwalker.track.service;

import com.example.trailwalker.track.R;

import trailwalker.track.ui.data.ApplicationSettingsInformation;
import android.content.Context;
import android.content.SharedPreferences;

/**
 * Classe que gestiona la persistncia de la configuraci de l'aplicaci
 * @author Albert Jord Vinardell
 * @version Desembre 2012
 * */
public class TWApplicationSetingsManager {
    //*******************************************************************************
    //Constants
    //*******************************************************************************
	public static final String PREFS_NAME = "TWAppPreferences";
	
	//Valors per defecte de l'aplicaci
	static final String REST_URL = "http://napols.uoc.es/arcgis/rest/services/TrailWalker/FeatureServer/0/addFeatures";
	final static long MIN_TIME_BETWEEN_LOC_UPDATES = 60*1000*10; //10 minuts
	final static long MIN_DIST_BETWEEN_LOC_UPDATES = 5; //5 metres
	final static boolean USE_NETWORK_LOCATION = true;
	final static boolean USE_GPS_LOCATION = true;
	final static boolean USE_GPS_FIRST = true;
	
    //*******************************************************************************
    //Variables
    //*******************************************************************************
	Context mContext = null;
	
    //*******************************************************************************
    //Constructor
    //*******************************************************************************
    /**
     * Constructor
     * @param context Objecte {@link Context} de l'aplicaci
     * */
	public TWApplicationSetingsManager(Context context){
		mContext = context;
	}
	
    //*******************************************************************************
    //Funcions
    //*******************************************************************************
	/**
	 * Funci per fixar els valors per defecte de configuraci de l'aplicaci 
	 * */
	public void setDefaultAppConfiguration(){
		long time_between_location_updates = MIN_TIME_BETWEEN_LOC_UPDATES;
		long distance_between_location_updates = MIN_DIST_BETWEEN_LOC_UPDATES;
		String restUrl = REST_URL;
		boolean useNetworkLocation = USE_NETWORK_LOCATION;
		boolean useGPSLocation = USE_GPS_LOCATION;
		boolean useGPSFirst = USE_GPS_FIRST;
		ApplicationSettingsInformation settings = new ApplicationSettingsInformation(time_between_location_updates, distance_between_location_updates, restUrl, useNetworkLocation, useGPSLocation, useGPSFirst);
		setAppConfiguration(settings);
	}
	
	/**
	 * Funci que torna els valors persistits de configuraci d'aplicaci
	 * @return instncia de {@link ApplicationSettingsInformation} amb la informaci de configuraci d'aplicaci
	 * */
	public ApplicationSettingsInformation getAppConfiguration(){
		SharedPreferences settings = mContext.getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE);
		ApplicationSettingsInformation appSettings = null;

		long time_between_location_updates = settings.getLong("time_between_location_updates",MIN_TIME_BETWEEN_LOC_UPDATES);
		long distance_between_location_updates = settings.getLong("distance_between_location_updates", MIN_DIST_BETWEEN_LOC_UPDATES);
		String restUrl = settings.getString("restUrl", REST_URL);
		boolean useNetworkLocation = settings.getBoolean("useNetworkLocation", USE_NETWORK_LOCATION);
		boolean useGPSLocation = settings.getBoolean("useGPSLocation", USE_GPS_LOCATION);
		boolean useGPSFirst = settings.getBoolean("useGPSFirst", USE_GPS_FIRST);
		appSettings = new ApplicationSettingsInformation(time_between_location_updates, distance_between_location_updates, restUrl, useNetworkLocation, useGPSLocation, useGPSFirst);
		return appSettings;
	}

	/**
	 * Funci per persistir els valors de configuraci de l'aplicaci provets
	 * @param settings instncia de {@link ApplicationSettingsInformation} que cont la informaci de configuraci
	 * */
	public void setAppConfiguration(ApplicationSettingsInformation settings){
		SharedPreferences appSettings = mContext.getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE);
		SharedPreferences.Editor editor = appSettings.edit();
		
		editor.putLong("time_between_location_updates", settings.getTime_between_location_updates());
		editor.putLong("distance_between_location_updates", settings.getDistance_between_location_updates());
		editor.putString("restUrl", settings.getRestUrl());
		editor.putBoolean("useNetworkLocation", settings.isUseNetworkLocation());
		editor.putBoolean("useGPSLocation", settings.isUseGPSLocation());
		editor.putBoolean("useGPSFirst", settings.isUseGPSFirst());
	      
		// Commit the edits!
		editor.commit();
	}

	/**
	 * Que torna el nom d'equip persistit
	 * @return {@link String} amb el nom d'equip
	 * */
	public String getTeamName(){
		SharedPreferences settings = mContext.getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE);
		String defaultTeamName = mContext.getString(R.string.sense_nom);
		return settings.getString("teamName", defaultTeamName);
	}
	
	/**
	 * Funci per persistir el nom de l'equip
	 * @param teamName {@link String} amb el nom de l'equip
	 * */
	public void setTeamName(String teamName){
		SharedPreferences appSettings = mContext.getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE);
		SharedPreferences.Editor editor = appSettings.edit();
		
		editor.putString("teamName", teamName);
		
		// Commit the edits!
		editor.commit();
	}
	
	/**
	 * Funci per fixar els valor per defecte del nom d'equip
	 * */
	public void setDefaultTeamName(){
		setTeamName(mContext.getString(R.string.sense_nom));
	}

}
