package trailwalker.track.service;

import java.util.ArrayList;
import java.util.Date;
import java.util.EventObject;

import com.example.trailwalker.track.R;

import trailwalker.track.data.LocationInformation;
import trailwalker.track.data.LocationInformationManager;
import trailwalker.track.device.DeviceInformationRetriever;
import trailwalker.track.eventHandling.LocationInformationManagerChangeStatusEvent;
import trailwalker.track.eventHandling.LocationInformationManagerChangeStatusEventListener;
import trailwalker.track.eventHandling.TWLocationManagerEvent;
import trailwalker.track.eventHandling.TWLocationManagerEventListener;
import trailwalker.track.location.TWLocationManager;
import trailwalker.track.ui.TrailWalkerTrackActivity;
import trailwalker.track.ui.data.ApplicationSettingsInformation;
import trailwalker.track.ui.data.StatusInformation;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.location.Location;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.os.Messenger;
import android.os.RemoteException;
import android.support.v4.app.NotificationCompat;
import android.util.Log;

public class TWService extends Service implements LocationInformationManagerChangeStatusEventListener,TWLocationManagerEventListener{
	
    //*******************************************************************************
    //Constants
    //*******************************************************************************
    public static final int MSG_REGISTER_CLIENT = 1;
    public static final int MSG_UNREGISTER_CLIENT = 2;
    public static final int MSG_MESSAG_STRING = 3;
    public static final int MSG_START_LOCATION_UPDATES = 4;
    public static final int MSG_STOP_LOCATION_UPDATES = 5;
    public static final int MSG_STATUS_INFORMATION = 6;
    public static final int MSG_GPS_NOT_ENABLED = 7;
    public static final int MSG_TEAM_NAME = 8;
    public static final int MSG_NETWORK_NOT_ENABLED = 9;
    public static final int MSG_LOCATION_RECEIVED = 10;
    public static final int MSG_SHOW_APPLICATION_SETTINGS = 11;
    public static final int MSG_APPLICATION_SETTINGS_CHANGED = 12;
    public static final int MSG_SHUTDOWN_APPLICATION = 13;
    public static final int MSG_CREATE_NOTIFICATION = 14;
    public static final int MSG_DESTROY_NOTIFICATION = 15;
    
    public static final int NOTIF_ID = 666999666;
    
    //*******************************************************************************
    //Variables
    //*******************************************************************************
    ArrayList<Messenger> mClients = new ArrayList<Messenger>(); // Array per gestionar tots els clients connectats a aquest servei
    final Messenger mMessenger = new Messenger(new IncomingHandler()); // Objecte per gestionar els "messages" amb l'"IncomingHandler"
	DeviceInformationRetriever deviceInformationRetriever = null;
	LocationInformationManager locationInformationManager = null;
	TWLocationManager twLocationManager = null;
	boolean requestingLocationUpdates = false;
	String teamName = "";
	
	NotificationManager mNotificationManager = null;
	NotificationCompat.Builder mNotifyBuilder = null;
	
    //*******************************************************************************
    //Cicle de Vida del Servei Android
    //*******************************************************************************
    @Override
    public void onCreate() {
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TWService: onCreate()");
    	super.onCreate();
    	initializeComponents();
    	//startRequestLocationUpdates();
    }
    
    @Override
    public void onDestroy() {
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TWService: onDestroy()");
    	unbindEventListeners();
    	this.stopRequestLocationUpdates();
    	try {
			deviceInformationRetriever.finalize();
		} catch (Throwable e) {
			Log.e("TrailWalkerTrack","TrailWalkerTrack: TWService: " + e.getMessage());
		}
        super.onDestroy();
    }
    
	@Override
	public IBinder onBind(Intent intent) {
		Log.d("TrailWalkerTrack","TrailWalkerTrack: TWService: onBind()");
		return mMessenger.getBinder();
	}
		
	@Override
    public int onStartCommand(Intent intent, int flags, int startId) {
		Log.d("TrailWalkerTrack","TrailWalkerTrack: TWService: onStartCommand()");
        return START_STICKY; // run until explicitly stopped.
    }
	
    //*******************************************************************************
    //Classes Inline
    //*******************************************************************************
    /**
     * Classe InLine per gestionar els missatges {@link Message}
     * */
	class IncomingHandler extends Handler {
        @Override
        public void handleMessage(Message msg) {
        	Log.d("TrailWalkerTrack","TrailWalkerTrack: TWService: IncomingHandler: HandleMessage()");
            switch (msg.what) {
            case MSG_REGISTER_CLIENT:
            	//Connexi per part de la TrailWalkertrackActivity
            	registerClient(msg);
                break;
            case MSG_UNREGISTER_CLIENT:
            	//Desconnexi per part de la TrailWalkertrackActivity
                unRegisterClient(msg);
                break;
            case MSG_START_LOCATION_UPDATES:
            	//Petici d'inici d'escolta de posicions
            	startRequestLocationUpdates();
            	break;
            case MSG_STOP_LOCATION_UPDATES:
            	//Petici d'aturada d'escolta de posicions
            	stopRequestLocationUpdates();
            	break;
            case MSG_STATUS_INFORMATION:
            	//Petici d'enviament d'informaci d'estat a la UI
            	sendToUIStatusInformation();
            	break;
            case MSG_TEAM_NAME:
            	//Notificaci de canvi de nom de l'equip
            	teamName = (String)msg.obj;
            	break;
            case MSG_APPLICATION_SETTINGS_CHANGED:
            	//Notificaci de canvi de configuraci de l'aplicaci
            	if(twLocationManager!=null&&locationInformationManager!=null){
            		twLocationManager.useSettings((ApplicationSettingsInformation)msg.obj);
            		locationInformationManager.useSettings((ApplicationSettingsInformation)msg.obj);
            	}
            	break;
            case MSG_CREATE_NOTIFICATION:
            	createNotification();
            	break;
            case MSG_DESTROY_NOTIFICATION:
            	destroyNotification();
            	break;
            default:
                super.handleMessage(msg);
            }
        }
    }
    
    //*******************************************************************************
    //Funcions
    //*******************************************************************************
	/**
	 * Funci per deixar d'escoltar els events {@link LocationInformationManagerChangeStatusEvent} i {@link TWLocationManagerEvent}
	 * */
	private void unbindEventListeners(){
		locationInformationManager.removeEventListener(this);
		twLocationManager.removeEventListener(this);
	}
	
	/**
	 * Funci per inicialitzar variables principals
	 * */
	private void initializeComponents(){
    	deviceInformationRetriever = new DeviceInformationRetriever(getBaseContext());;
    	ApplicationSettingsInformation settings = new TWApplicationSetingsManager(getBaseContext()).getAppConfiguration();
    	locationInformationManager = new LocationInformationManager(settings);
    	locationInformationManager.addEventListener(this);
    	twLocationManager = new TWLocationManager(getBaseContext(), settings);
    	twLocationManager.addEventListener(this);
	}
	
	/**
	 * Funci per indicar al {@link TWLocationManager} que inici la captura de posicions
	 * */
	private void startRequestLocationUpdates(){
		Log.d("TrailWalkerTrack","TrailWalkerTrack: TWService: startRequestLocationUpdates()");
		twLocationManager.startLocationUpdates();
	}
	
	/**
	 * Funci per indicar al {@link TWLocationManager} que aturi la captura de posicions
	 * */
	private void stopRequestLocationUpdates(){
		Log.d("TrailWalkerTrack","TrailWalkerTrack: TWService: stopRequestLocationUpdates()");
		twLocationManager.stopLocationUpdates();
	}
	
	/**
	 * Funci per enregistrar un client {@link TrailWalkerTrackActivity}, quan es connecta al servei
	 * @param msg Message que s'utilitzar per enviar la notificaci del canvi a la {@link TrailWalkerTrackActivity}
	 * */
	private void registerClient(Message msg){
		Log.d("TrailWalkerTrack","TrailWalkerTrack: TWService: registerClient()");
        mClients.add(msg.replyTo);
        sendToUIStatusInformation();
	}

	/**
	 * Funci per desregistrar un client {@link TrailWalkerTrackActivity}
	 * @param msg {@link Message} que s'utilitzar per enviar la notificaci del canvi a la {@link TrailWalkerTrackActivity}
	 * */
	private void unRegisterClient(Message msg){
		Log.d("TrailWalkerTrack","TrailWalkerTrack: TWService: unRegisterClient()");
		mClients.remove(msg.replyTo);
	}
	
	/**
	 * Funci per informar a la UI si s'estan capturant posicions o no
	 * @param b true si s'estan capturant posicions i false si no .
	 * */	
	private void setRequestingLocationUpdates(boolean b){
		requestingLocationUpdates = b;
		sendToUIStatusInformation();
	}
	
	/**
	 * Funci que gestiona una posici capturada i la passa al {@link LocationInformationManager} per a que l'emmagatzemi i l'envi al servidor GIS
	 * @param loc instncia de {@link Location} amb la informaci de la posici
	 * */
    private void storeLocationInformation(Location loc){
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TWService: storeLocationInformation()");
    	LocationInformation locI = new LocationInformation(loc, "102100", deviceInformationRetriever.getIMSI(), teamName, deviceInformationRetriever.getBatteryLevelPercentage());
    	locationInformationManager.storeAndSendLocationToServer(locI);
    }
    
    /**
     * Funci per enviar informaci o acci a la UI {@link TrailWalkerTrackActivity}
     * @param messageCode enter que indica el tipus de missatge. Constants definides a {@link TWService}
     * @param statusInformation instncia de {@link StatusInformation} que cont la informaci d'estat de l'aplicaci
     * */
    private void sendMessageToUI(int messageCode, StatusInformation statusInformation) {
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TWService: sendMessageToUI()");
        for (int i=mClients.size()-1; i>=0; i--) {
            try {
            	Message mes = new Message();
            	mes.what = messageCode;
            	mes.obj=statusInformation;
            	mClients.get(i).send(mes);
            } catch (RemoteException e) {
                mClients.remove(i);
            }
        }
    }
    
    /**
     * Funci per enviar la informaci d'estat a la UI
     * */
    private void sendToUIStatusInformation(){
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TWService: sendToUIStatusInformation()");
    	StatusInformation si = new StatusInformation(requestingLocationUpdates,locationInformationManager.getNumberOfLocationInformationSent(),locationInformationManager.getNumberOfLocationInformationStored(),"");
    	sendMessageToUI(MSG_STATUS_INFORMATION,si);
    	updateNotification();
    }
    
    /**
     * Funci per indicar a la UI que el GPS no est activat
     * */
    private void sendToUIGPSNotEnabled(){
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TWService: sendToUIGPSNotEnabled()");
    	sendMessageToUI(MSG_GPS_NOT_ENABLED, null);
    }
    
    /**
     * Funci per indicar a la UI que les xarxes mbils (Localitzaci) no estan activades
     * */
    private void sendToUINetworkNotEnabled(){
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TWService: sendToUINetworkNotEnabled()");
    	sendMessageToUI(MSG_NETWORK_NOT_ENABLED, null);
    }

    /**
     * Implementaci de handleLocationInformationManagerChangeStatusEvent, per gestionar els events {@link LocationInformationManagerChangeStatusEvent}
     * */
	@Override
	public void handleLocationInformationManagerChangeStatusEvent(EventObject e) {
		Log.d("TrailWalkerTrack","TrailWalkerTrack: TWService: handleLocationInformationManagerChangeStatusEvent()");
		sendToUIStatusInformation();
	}

    /**
     * Implementaci de handleTWLocationManagerEvent, per gestionar els events {@link TWLocationManagerEvent}
     * */
	@Override
	public void handleTWLocationManagerEvent(TWLocationManagerEvent e) {
		Message msg = e.getMessage();
		switch (msg.what) {
	        case MSG_START_LOCATION_UPDATES:
	        	//S'ha iniciat la captura de posicions
	        	setRequestingLocationUpdates(true);
	            break;
	        case MSG_STOP_LOCATION_UPDATES:
	        	//S'ha aturat la captura de posicions
	        	setRequestingLocationUpdates(false);
	            break;
	        case MSG_GPS_NOT_ENABLED:
	        	//El GPS no est activat
	        	sendToUIGPSNotEnabled();
	            break;
	        case MSG_NETWORK_NOT_ENABLED:
	        	//Les xarxes mbils (Localitzaci) no estan activades
	        	sendToUINetworkNotEnabled();
	            break;
	        case MSG_LOCATION_RECEIVED:
	        	//Es reb una nova localitzaci
	        	Location loc = (Location)msg.obj;
	        	storeLocationInformation(loc);
	        	break;
        }
	}
	
	/**
	 * Destrueix la notificaci de l'aplicaci
	 * */
    public void destroyNotification(){
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: destroyNotification()");
    	mNotificationManager.cancel(NOTIF_ID);
    	mNotificationManager = null;
    	mNotifyBuilder = null;
    }
    
    public void updateNotification(){
    	if(mNotifyBuilder!=null){
    		new Thread(
    			    new Runnable() {
    			        @Override
    			        public void run() {
    			        	String sentLocations = ""+locationInformationManager.getNumberOfLocationInformationSent();
    			        	String pendingToSendLocations = ""+locationInformationManager.getNumberOfLocationInformationStored();
    			        	String capturedLocations = ""+(locationInformationManager.getNumberOfLocationInformationSent()+locationInformationManager.getNumberOfLocationInformationStored());
    			        	
    			        	mNotifyBuilder.setContentText(getString(requestingLocationUpdates?R.string.toggleButtonGPSServiceON:R.string.toggleButtonGPSServiceOFF));
    			        	mNotifyBuilder.setContentInfo(sentLocations+"|"+capturedLocations+"|"+pendingToSendLocations);
    			        	Date date = new Date();
    			        	mNotifyBuilder.setWhen(date.getTime());
    			        	
    			        	mNotificationManager.notify(NOTIF_ID, mNotifyBuilder.build());
    			        }
    			    }
    			).start();
    	}
    }
    
    /**
     * Crea la notificaci de l'aplicaci
     * */
    public void createNotification(){
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: createNotification()");
    	mNotifyBuilder =
    	        new NotificationCompat.Builder(this)
    	        .setSmallIcon(R.drawable.ic_launcher)
    	        .setContentTitle(getString(R.string.title_activity_trail_walker_track));
    	
		Intent intent=new Intent(this, TrailWalkerTrackActivity.class);
		intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP|Intent.FLAG_ACTIVITY_SINGLE_TOP);
		PendingIntent pi=PendingIntent.getActivity(this, 0, intent, 0);
		
    	mNotifyBuilder.setContentIntent(pi);
    	
    	mNotificationManager =
    	    (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);
    	// mId allows you to update the notification later on.
    	//mNotificationManager.notify(NOTIF_ID, mNotifyBuilder.build());
    	updateNotification();
    }
    
	/**
	 * Destructor de la classe
	 * */
    @Override
	public void finalize() throws Throwable{
		destroyNotification();
		super.finalize();
	} 
	
}
