package trailwalker.track.ui;

import trailwalker.track.service.TWApplicationSetingsManager;
import trailwalker.track.ui.data.ApplicationSettingsInformation;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.graphics.Color;
import android.os.Message;
import android.text.InputType;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.ScrollView;
import android.widget.TableLayout;
import android.widget.TableRow;
import android.widget.TextView;

import com.example.trailwalker.track.R;

/**
 * Classe per mostrar una caixa de dileg per modificar la configuraci de l'aplicaci
 * @author Albert Jord Vinardell
 * @version Desembre 2012
 * */
public class ApplicationSettingsDialog {
    //*******************************************************************************
    //Variables
    //*******************************************************************************	
	private Context mContext = null;
	private AlertDialog.Builder alertDialog = null;
	LayoutInflater inflater = null;
	View layout = null;
	
	private EditText txtTempsEntreLocalitzacionsValor = null;
	private EditText txtDistanciaEntreLocalitzacionsValor = null;
	private EditText txtURLRestApiValor = null;
	private RadioButton rbSistemaLocalitzacioGPSValor = null;
	private RadioButton rbSistemaLocalitzacioXarxaValor = null;
	private RadioButton rbSistemaLocalitzacioGPSXarxaValor = null;
	private RadioButton rbSistemaLocalitzacioXarxaGPSValor = null;
	private Message message;

    //*******************************************************************************
    //Constructor
    //*******************************************************************************
    /**
     * Constructor
     * @param context Objecte {@link Context} de l'aplicaci
     * @param message Objecte {@link Message} que tonar a la {@link TrailWalkerTrackActivity} per indicar el nou nom
     * */
	public ApplicationSettingsDialog(Context context, Message message){
		mContext = context;
		this.message = message;
	}

    //*******************************************************************************
    //Funcions
    //*******************************************************************************
	/**
	 * Funci per crear l'estructura UI de la caixa de dileg
	 * */
	private void createStructure(){
		ScrollView sv = new ScrollView(mContext);
		
		sv.setBackgroundColor(Color.argb(255, 255, 255, 255));
		
		TableLayout tl = new TableLayout(mContext);
		TableLayout.LayoutParams lp = new TableLayout.LayoutParams(
			ViewGroup.LayoutParams.MATCH_PARENT,
			ViewGroup.LayoutParams.MATCH_PARENT);
		tl.setLayoutParams(lp);
		tl.setStretchAllColumns(true);
		
		TableLayout.LayoutParams rowLp = new TableLayout.LayoutParams(
		        ViewGroup.LayoutParams.MATCH_PARENT,
		        ViewGroup.LayoutParams.MATCH_PARENT,
		        1.0f);
		TableRow.LayoutParams cellLp = new TableRow.LayoutParams(
		        ViewGroup.LayoutParams.MATCH_PARENT,
		        ViewGroup.LayoutParams.MATCH_PARENT,
		        1.0f);
		
		sv.addView(tl);
		
		TableRow tr = new TableRow(mContext);
		tl.addView(tr, rowLp);
		TextView tv = new TextView(mContext);
		tr.addView(tv, cellLp);
		tv.setText(((Activity)mContext).getString(R.string.temps_entre_captures_texte));
		tr = new TableRow(mContext);
		tl.addView(tr, rowLp);
		txtTempsEntreLocalitzacionsValor = new EditText(mContext);
		txtTempsEntreLocalitzacionsValor.setInputType(InputType.TYPE_CLASS_NUMBER);
		txtTempsEntreLocalitzacionsValor.setTextAppearance(mContext, android.R.attr.textAppearanceSmall);
		tr.addView(txtTempsEntreLocalitzacionsValor, cellLp);

		tr = new TableRow(mContext);
		tl.addView(tr, rowLp);
		tv = new TextView(mContext);
		tr.addView(tv, cellLp);
		tv.setText(R.string.distancia_entre_captures);
		tr = new TableRow(mContext);
		tl.addView(tr);
		txtDistanciaEntreLocalitzacionsValor = new EditText(mContext);
		txtDistanciaEntreLocalitzacionsValor.setInputType(InputType.TYPE_CLASS_NUMBER);
		txtDistanciaEntreLocalitzacionsValor.setTextAppearance(mContext, android.R.attr.textAppearanceSmall);
		tr.addView(txtDistanciaEntreLocalitzacionsValor, cellLp);
		
		tr = new TableRow(mContext);
		tl.addView(tr, rowLp);
		tv = new TextView(mContext);
		tr.addView(tv, cellLp);
		tv.setText(R.string.url_api_rest);
		tr = new TableRow(mContext);
		tl.addView(tr, rowLp);
		txtURLRestApiValor = new EditText(mContext);
		txtURLRestApiValor.setInputType(InputType.TYPE_TEXT_VARIATION_URI|InputType.TYPE_TEXT_FLAG_MULTI_LINE);
		txtURLRestApiValor.setTextAppearance(mContext, android.R.attr.textAppearanceSmall);
		tr.addView(txtURLRestApiValor, cellLp);
		
		
		tr = new TableRow(mContext);
		tl.addView(tr, rowLp);
		RadioGroup rg = new RadioGroup(mContext);
		tr.addView(rg, cellLp);

		rbSistemaLocalitzacioGPSValor = new RadioButton(mContext);
		rg.addView(rbSistemaLocalitzacioGPSValor);
		rbSistemaLocalitzacioGPSValor.setText(R.string.gps);
		
		rbSistemaLocalitzacioXarxaValor = new RadioButton(mContext);
		rg.addView(rbSistemaLocalitzacioXarxaValor);
		rbSistemaLocalitzacioXarxaValor.setText(R.string.xarxa);
		
		rbSistemaLocalitzacioGPSXarxaValor = new RadioButton(mContext);
		rg.addView(rbSistemaLocalitzacioGPSXarxaValor);
		rbSistemaLocalitzacioGPSXarxaValor.setText(R.string.gps_i_xarxa);
		
		rbSistemaLocalitzacioXarxaGPSValor = new RadioButton(mContext);
		rg.addView(rbSistemaLocalitzacioXarxaGPSValor);
		rbSistemaLocalitzacioXarxaGPSValor.setText(R.string.xarxa_i_gps);
		
		tr = new TableRow(mContext);
		tl.addView(tr, rowLp);
		Button button = new Button(mContext);
		tr.addView(button, cellLp);
		button.setText(R.string.valors_per_defecte);
		button.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				new TWApplicationSetingsManager(mContext).setDefaultAppConfiguration();
				fillInformation();
				//rbSistemaLocalitzacioGPSXarxaValor.setChecked(true);
			}
		});
		
		alertDialog.setView(sv);
	}

	/**
	 * Funci per recuperar la informaci persistida de la configuraci d'aplicaci i omplir els controls de UI
	 * */
	public void fillInformation(){
		
    	ApplicationSettingsInformation settings = new TWApplicationSetingsManager(mContext).getAppConfiguration();
    	    	
    	txtTempsEntreLocalitzacionsValor.setText(""+(settings.getTime_between_location_updates()/1000));
    	txtDistanciaEntreLocalitzacionsValor.setText(""+settings.getDistance_between_location_updates());
    	txtURLRestApiValor.setText(""+settings.getRestUrl());
    	
    	boolean gps = settings.isUseGPSLocation();
    	boolean net = settings.isUseNetworkLocation();
    	boolean useGPSFirst = settings.isUseGPSFirst();
   	
    	if(gps&&net){
    		if(useGPSFirst)rbSistemaLocalitzacioGPSXarxaValor.setChecked(true);
    		else rbSistemaLocalitzacioXarxaGPSValor.setChecked(true);
    	}
    	else{
    		if(gps)rbSistemaLocalitzacioGPSValor.setChecked(true);
    		if(net)rbSistemaLocalitzacioXarxaValor.setChecked(true);
    	}
    	
	}
	
    /**
     * Funci per mostrar la caixa de dileg
     * */
	public void show(){
		alertDialog = new AlertDialog.Builder(mContext);
		alertDialog.setTitle(R.string.configuracio_aplicacio);
		alertDialog.setIcon(R.drawable.ic_launcher);
		
		setPositiveButton();
		setNegativeButton();
    	
		createStructure();
		fillInformation();
    	    	
		alertDialog.show();

	}
	
    /**
     * Implementaci del bot OK
     * */
	private void setPositiveButton(){
		alertDialog.setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog,int which) {

            	long time_between_location_updates = Long.valueOf(txtTempsEntreLocalitzacionsValor.getText().toString())*1000;
            	long distance_between_location_updates = Long.valueOf(txtDistanciaEntreLocalitzacionsValor.getText().toString());
            	String restUrl = txtURLRestApiValor.getText().toString();
            	
            	boolean useGPSFirst = false;
            	boolean useGPSLocation = rbSistemaLocalitzacioGPSValor.isChecked();
            	boolean useNetworkLocation = rbSistemaLocalitzacioXarxaValor.isChecked();
            	
            	if(rbSistemaLocalitzacioGPSXarxaValor.isChecked()){
            		useGPSLocation = true;
            		useNetworkLocation = true;
            		useGPSFirst = true;
            	}
            	
            	if(rbSistemaLocalitzacioXarxaGPSValor.isChecked()){
            		useGPSLocation = true;
            		useNetworkLocation = true;
            	}
            	
            	ApplicationSettingsInformation settings = new ApplicationSettingsInformation(time_between_location_updates, distance_between_location_updates, restUrl, useNetworkLocation, useGPSLocation, useGPSFirst);
            	//Persistir la informaci de configuraci
            	new TWApplicationSetingsManager(mContext).setAppConfiguration(settings);
            	message.obj = settings;
            	//Enviar a la TrailWalkerTrackActivity la notificaci dels canvis
            	message.sendToTarget();
            }
        });
	}
	
    /**
     * Implementaci del bot cancelar
     * */
	private void setNegativeButton(){
		alertDialog.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int which) {
            dialog.cancel();
            }
        });
	}

}
