package trailwalker.track.ui;

import trailwalker.track.service.TWService;

import com.example.trailwalker.track.R;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.provider.Settings;
import android.util.Log;

/**
 * Classe per mostrar una caixa de dileg per avisar a l'usuari que les xarxes mbils o el GPS estan desactivats
 * @author Albert Jord Vinardell
 * @version Desembre 2012
 * */
public class LocationSettingsAlert {
    //*******************************************************************************
    //Variables
    //*******************************************************************************
	private Context mContext;
	private boolean isGPSAlert = false;
    
    //*******************************************************************************
    //Constructor
    //*******************************************************************************
    /**
     * Constructor
     * @param context Objecte {@link Context} de l'aplicaci
     * */
	public LocationSettingsAlert(Context context){
    	this.mContext=context;
    }
    
    //*******************************************************************************
    //Funcions
    //*******************************************************************************
    /**
     * Funci per mostrar la caixa de dileg
     * @param isGPSAlert boole que indica si l'alerta s per GPS (true) o s per xarxes mbils (false)
     * */
	public void show(boolean isGPSAlert){
		Log.d("TrailWalkerTrack","TrailWalkerTrack: LocationSettingsAlert: show() --> " + isGPSAlert);
		this.isGPSAlert = isGPSAlert;
		
		int string;
		
        AlertDialog.Builder alertDialog = new AlertDialog.Builder(mContext);
 
        // Setting Dialog Title
        string = (isGPSAlert)?R.string.gps_desactivats:R.string.xarxes_desactivats;
        alertDialog.setTitle(string);
        
        alertDialog.setIcon(R.drawable.ic_launcher);
 
        // Setting Dialog Message
        string = (isGPSAlert)?R.string.gps_desactivats_texte:R.string.xarxes_desactivats_texte;
        alertDialog.setMessage(string);
 
        // Implementaci del bot OK. L'usuari ser redirigit a la pantalla de configuraci de serveis de localitzaci d'Android
        alertDialog.setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog,int which) {
                showLocationSettings(true);
            }
        });
 
        // Implementaci del bot Cancelar
        alertDialog.setNegativeButton(mContext.getString(R.string.cancel), new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int which) {
            dialog.cancel();
            }
        });
 
        // Showing Alert Message
        alertDialog.show();
    }
	
	/**
	 * Funci per mostrar la pantalla de configuraci de serveis de localitzaci d'Android
	 * */
    public void showLocationSettings(boolean internalCall){
        Intent intent = new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
        ((Activity)mContext).startActivityForResult(intent, (this.isGPSAlert)?TWService.MSG_GPS_NOT_ENABLED:TWService.MSG_NETWORK_NOT_ENABLED);
    }
    
    public void showLocationSettings(){
    	showLocationSettings(false);
    }
}
