package trailwalker.track.ui;

import java.text.SimpleDateFormat;
import java.util.Date;

import com.example.trailwalker.track.R;

import trailwalker.track.service.TWApplicationSetingsManager;
import trailwalker.track.service.TWService;
import trailwalker.track.ui.data.StatusInformation;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.os.Messenger;
import android.os.RemoteException;
import android.app.Activity;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.support.v4.app.NotificationCompat;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.ToggleButton;

/**
 * Classe que hereda de {@link Activity} que cont la lgica de la UI
 * @author Albert Jord Vinardell
 * @version Desembre 2012
 * */
public class TrailWalkerTrackActivity extends Activity {

    //*******************************************************************************
    //Variables
    //*******************************************************************************
	boolean mIsBound = false;
	Messenger mService = null;
	final Messenger mMessenger = new Messenger(new IncomingHandler());
	Context context = this;
	private ServiceConnection mConnection = null;
	private ToggleButton bToggleButtonGPSService = null;
	private TextView tPosicionsEnviades = null;
	private TextView tPosicionsCapturades = null;
	private TextView tPosicionsPendentsEnviar = null;
	private TextView tDataHoraActualitzacio = null;
	private TextView tTeamName = null;
	
    //*******************************************************************************
    //Cicle de vida de Activity
    //*******************************************************************************
	@Override
    public void onCreate(Bundle savedInstanceState) {
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: onCreate()");
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_trail_walker_track);
        initializeUI();
        retrieveStoredTeamName();
        initializeServiceConnection();
    }
    
    @Override
    public void onStart(){
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: onStart()");
    	super.onStart();
    	this.doBindService();
    }

    @Override
    public void onResume(){
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: onResume()");
    	super.onResume();
    	destroyNotification();
    }
    
    @Override
    public void onPause(){
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: onPause()");
    	super.onPause();
    	createNotification();
    }
    
    @Override
    public void onDestroy(){
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: onDestroy()");
    	doUnbindService();
    	super.onDestroy();
    	destroyNotification();
    }
    
    //*******************************************************************************
    //Gesti Events
    //******************************************************************************

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.activity_trail_walker_track, menu);
        return true;
    }
    
    /**
     * Funci per gestionar la selleci d'opcions del men
     * */
	public boolean onOptionsItemSelected(MenuItem item) {
		Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: onOptionsItemSelected()");
		switch(item.getItemId()){
		case R.id.serveis_ubicacio:
			//Mostrar la pantalla d'Android de gesti dels serveis de localitzaci
			showLocationSettings();
			break; 
		case R.id.tancar_aplicacio:
			//L'usuari vol tancar l'aplicaci. Se li demana confirmaci.
			askUserToConfirmShutDownApplication();
			break;
		case R.id.modificar_nom_equip:
			//L'usuari vol modificar el nom de l'equip
			changeTeamName();
			break;
		case R.id.configuracio_aplicacio:
			//L'usuari vol modificar la configuraci de l'aplicaci
			showApplicationSettingsIni();
			break;
		default:
			break;
		}
		return true;
	}
	
    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
		Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: onKeyDown()");
		
		boolean b = false;
		//Es controla que si la tecla premuda s "Back o enrere", l'aplicaci no es destrueixi, sin que passi a un segon pla (moveTaskToBack(true))
		if (keyCode == KeyEvent.KEYCODE_BACK) {
			moveTaskToBack(true);
			b = true;
		}
		else b = super.onKeyDown(keyCode, event);
		return b;
	}
    
    /**
     * Funci que sobreescriu la funci onWIndowFocusChanged de la classe {@link Activity}, de forma que cada cop que es rebi el focus, s'actualitzi la informaci de l'aplicaci
     * (posicions enviades...)
     * */
    @Override
    public void onWindowFocusChanged(boolean hasFocus){
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: onWindowFocusChanged()");
    	if(hasFocus)sendMessagetoService(TWService.MSG_STATUS_INFORMATION, null);
    }
    
    /**
     * Funci per controlar l'event click del bot ON/OFF per engegar i aturar el servei de captura de posicions
     * */
    public void onClickToggleButtonGPSService(View view){
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: onClickToggleButtonGPSService(): " + bToggleButtonGPSService.isChecked());
    	if(bToggleButtonGPSService.isChecked())tellServiceStartRequestLocationUpdates();
    	else tellServiceStopRequestLocationUpdates();
    }
    
    /**
     * Funci per controlar el callback dels Intent per obrir altres Activity
     * */
    @Override
    protected void onActivityResult (int requestCode, int resultCode, Intent data){
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: onActivityResult()");
    	//Si estic tornant de la pgina de configuraci dels serveis d'ubicaci, torno a intentar comenar a capturar posicions
    	if(requestCode==TWService.MSG_GPS_NOT_ENABLED||requestCode==TWService.MSG_NETWORK_NOT_ENABLED)tellServiceStartRequestLocationUpdates();
    }
    
    //*******************************************************************************
    //Classes Inline
    //******************************************************************************
    /**
     * Classe InLine per gestionar els missatges {@link Message}
     * */
    class IncomingHandler extends Handler {
        @Override
        public void handleMessage(Message msg) {
        	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: IncomingHandler: handleMessage()");
        	switch (msg.what) {
        	case TWService.MSG_TEAM_NAME:
        		//Canvi de nom d'equip
        		setTeamName((String)msg.obj);
        		break;
        	case TWService.MSG_GPS_NOT_ENABLED:
        		//Mostrar caixa de dileg perqu el GPS no est activat
        		showGPSNotEnabledAlert();
        		break;
        	case TWService.MSG_NETWORK_NOT_ENABLED:
        		//Mostrar caixa de dileg perqu les xarxes mbils no estan activades (localitzaci)
        		showNetworkNotEnabledAlert();
        		break;
        	case TWService.MSG_STATUS_INFORMATION:
        		//Actualitzaci de l'estat de l'aplicaci en UI
        		StatusInformation si = (StatusInformation)(msg.obj);
        		
        		String sent = ""+(si.getNumberOfLocationInformationSent());
        		String captured = ""+(si.getNumberOfLocationInformationStored()+si.getNumberOfLocationInformationSent());
        		String pending = ""+(si.getNumberOfLocationInformationStored());
        		
        		tPosicionsCapturades.setText(captured);
        		tPosicionsEnviades.setText(sent);
        		tPosicionsPendentsEnviar.setText(pending);
        		tDataHoraActualitzacio.setText(getCurrentDateTime());
        		        		
        		boolean b = si.isRequestingLocationUpdates();
        		bToggleButtonGPSService.setChecked(b);
        		
        		break;
        	case TWService.MSG_SHOW_APPLICATION_SETTINGS:
        		//Mostrar caixa de dileg de la configuraci de l'aplicaci
        		showApplicationSettings();
        		break;
        	case TWService.MSG_APPLICATION_SETTINGS_CHANGED:
        		//Enviar informaci o una acci al TWService
        		sendMessagetoService(msg.what, msg.obj);
        		break;
        	case TWService.MSG_SHUTDOWN_APPLICATION:
        		//Tancar l'aplicaci
        		shutdownApplication();
        		break;
            default:
                super.handleMessage(msg);
            }
        }
    }
    
    //*******************************************************************************
    //Funcions
    //******************************************************************************
    /**
     * Funci que avisa al TWService que aturi la captura de posicions
     * */
    private void tellServiceStopRequestLocationUpdates(){
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: tellServiceStopRequestLocationUpdates()");
    	sendMessagetoService(TWService.MSG_STOP_LOCATION_UPDATES, null);
    }
    
    /**
     * Funci que avisa al TWService que comenci la captura de posicions
     * */    
    private void tellServiceStartRequestLocationUpdates(){
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: tellServiceStartRequestLocationUpdates()");
    	sendMessagetoService(TWService.MSG_START_LOCATION_UPDATES, null);
    }
    
    /**
     * Funci que realitza la connexi amb el {@link TWService}. En la connexi el {@link TWService} inicia el seu cicle de vida. 
     * */
    void doBindService() {
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: doBindService()");
        bindService(new Intent(this, TWService.class), mConnection, Context.BIND_AUTO_CREATE);
        mIsBound = true;
    }

    /**
     * Funci que finalitza la connexi amb el {@link TWService}. 
     * */    
    void doUnbindService() {
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: doUnbindService()");
        if (mIsBound) {
            unbindService(mConnection);
            mIsBound = false;
        }
    }
    
    /*
    void startService(){
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: startService()");
    	startService(new Intent(TrailWalkerTrackActivity.this, TWService.class));
    }
    */
    
    /**
     * Funci que recupera el nom d'equip persistit i li passa a la funci "setTeamName"
     * */
    private void retrieveStoredTeamName(){
    	TWApplicationSetingsManager asm = new TWApplicationSetingsManager(this);
    	String teamName = asm.getTeamName();
    	setTeamName(teamName);
    }
    
    /**
     * Funci per mostrar un dileg per demanar la confirmaci de tancament de l'aplicaci a l'usuari
     * */
    private void askUserToConfirmShutDownApplication(){
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: askUserToConfirmShutDownApplication()");
    	//Es crea un Message que, en cas de que torni (en el sda) el gestionar el "new IncomingHandler()"
	    final Message message = Message.obtain();
	    message.setTarget(new IncomingHandler());
	    message.what = TWService.MSG_SHUTDOWN_APPLICATION;
	    ShutDownApplicationDialog sda = new ShutDownApplicationDialog(this, message);
	    sda.show();
    }
    
    /**
     * Funci per realitzar un tancament controlat de l'aplicaci.
     * */
    public void shutdownApplication(){
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: shutdownApplication()");
    	Toast.makeText(this, R.string.adeu, Toast.LENGTH_LONG).show();
    	//Desconnecta el servei
    	doUnbindService();
    	//Tanca l'aplicaci
    	this.finish();
    }
    
    /**
     * Funci que inicialitza els objectes per gestionar la UI
     * */
    private void initializeUI(){
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: initializeUI()");
    	bToggleButtonGPSService = (ToggleButton)findViewById(R.id.toggleButtonGPSService);
    	tPosicionsEnviades = (TextView)findViewById(R.id.posicionsEnviadesValor);
    	tPosicionsCapturades = (TextView)findViewById(R.id.posicionsCapturadesValor);
    	tPosicionsPendentsEnviar = (TextView)findViewById(R.id.posicionsPendentsEnviarValor);
    	tDataHoraActualitzacio = (TextView)findViewById(R.id.dataHoraActualitzacio);
    	tTeamName = (TextView)findViewById(R.id.equipValor);
    }
    
    /**
     * Funci que torna el dia i hora actual
     * */
    private String getCurrentDateTime(){
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: getCurrentDateTime()");
		SimpleDateFormat dateFormat = new SimpleDateFormat("dd/MM/yy HH:mm:ss");
		Date date = new Date();
		return dateFormat.format(date);
    }
    
    /**
     * Funci que crea inicialitza la connexi amb el {@link TWService}. Tamb s'implementen les accions a realitzar en el cas de que es realitzi una desconnexi
     * */
    private void initializeServiceConnection(){
    	mConnection = new ServiceConnection() {
    		// Implementaci de l'event "onServiceConnected" en cas de connexi amb el servei
            public void onServiceConnected(ComponentName className, IBinder service) {
                mService = new Messenger(service);
                Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: ServiceConnection: onServiceConnected()");
                try {
                    Message msg = Message.obtain(null, TWService.MSG_REGISTER_CLIENT);
                    msg.replyTo = mMessenger;
                    mService.send(msg);
                    //Envio al TWService el nom de l'equip
                    sendMessagetoService(TWService.MSG_TEAM_NAME, tTeamName.getText().toString());
                    //Comunico al servei que comenci a capturar posicions
                    sendMessagetoService(TWService.MSG_START_LOCATION_UPDATES, null);
                    
                } catch (RemoteException e) {
                	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: ServiceConnection: onServiceConnected: RemoteException: " + e.getMessage());
                }
            }
            
            // Implementaci de l'event "onServiceDisconnected" en cas de desconnexi amb el servei
            public void onServiceDisconnected(ComponentName className) {
                mService = null;
                bToggleButtonGPSService.setEnabled(false);
                Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: ServiceConnection: onServiceDisConnected()");
            }
        };
    }

    /**
     * Funci per enviar una acci o informaci al {@link TWService} (comunicaci amb {@link TWService})
     * @param messageCode codi enter identificador de l'acci o informaci. Constants definides a la classe {@link TWService}
     * @param obj informaci a enviar a {@link TWService}
     * */
    private void sendMessagetoService(int messageCode, Object obj){
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: sendMessagetoService()");

    	if(mService==null){
    		Log.e("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: sendMessagetoService(): Service not binded. Unable to interact with the service.");
    		return;
    	}    			
    			
    	Message mes = new Message();
    	mes.what = messageCode;
    	mes.obj=obj;
    	try {
			mService.send(mes);
		} catch (RemoteException e) {
			Log.e("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: sendMessagetoService(): Exception: " + e.getMessage());		
		}
    }
    
    /**
     * Funci per mostrar la pantalla d'Android de gesti dels serveis de localitzaci
     * */
    public void showLocationSettings(){
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: showLocationSettings()");
    	LocationSettingsAlert gsa = new LocationSettingsAlert(context);
    	gsa.showLocationSettings();
    }
    
    /**
     * Funci per mostrar una caixa de dileg indicant que el GPS no est activat.
     * */
    private void showGPSNotEnabledAlert(){
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: showGPSNotEnabledAlert()");
    	boolean isGPS = true;
    	(new LocationSettingsAlert(context)).show(isGPS);
    }
    
    /**
     * Funci per mostrar una caixa de dileg indicant que les xarxes mbils (localitzaci) no estan activades
     * */
    private void showNetworkNotEnabledAlert(){
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: showNetworkNotEnabledAlert()");
    	boolean isGPS = false;
    	(new LocationSettingsAlert(context)).show(isGPS);
    }
    
    /**
     * Funci per mostrar una caixa de dileg per canviar el nom de l'equip
     * */
    private void changeTeamName(){
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: changeTeamName()");
    	//Es crea un Message que el gestionaria el "new IncomingHandler()"
	    final Message message = Message.obtain();
	    message.setTarget(new IncomingHandler());
	    message.what = TWService.MSG_TEAM_NAME;
	    message.obj = tTeamName.getText().toString();
    	new TeamSettingsDialog(this,message).show();
    }
    
    /**
     * Funci que canvia el nom d'equip en la UI i envia al TWService el canvi per a que el tingui en compte
     * @param teamName nom de l'equip
     * */
    public void setTeamName(String teamName){
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: setTeamName()");
    	tTeamName.setText(teamName);
    	sendMessagetoService(TWService.MSG_TEAM_NAME, teamName);
    }
    
    /**
     * Funci per mostrar una caixa de dileg amb la configuraci de l'aplicaci
     * */
    public void showApplicationSettings(){
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: showApplicationSettings()");
    	//Es crea un Message que el gestionaria el "new IncomingHandler()"
	    final Message message = Message.obtain();
	    message.setTarget(new IncomingHandler());
	    message.what = TWService.MSG_APPLICATION_SETTINGS_CHANGED;
    	ApplicationSettingsDialog applicationSettingsDialog = new ApplicationSettingsDialog(this, message);
    	applicationSettingsDialog.show();
    }
    
    /**
     * Funci que genera una caixa de dileg per notificar a l'usuari del perill de manipular la configuraci de l'aplicaci
     * */
    public void showApplicationSettingsIni(){
    	Log.d("TrailWalkerTrack","TrailWalkerTrack: TrailWalkerTrackActivity: showApplicationSettingsIni()");
    	//Es crea un Message que el gestionaria el "new IncomingHandler()"
    	final Message message = Message.obtain();
	    message.setTarget(new IncomingHandler());
	    message.what = TWService.MSG_SHOW_APPLICATION_SETTINGS;
    	new ApplicationSettingsDialogIni(this, message).show();
    }
    
    /**
     * Envia un missatge al TWService per a que destrueixi la notificaci
     * */
    private void destroyNotification(){
    	sendMessagetoService(TWService.MSG_DESTROY_NOTIFICATION,null);
    }
    
    /**
     * Envia un missatge al TWService per a que crei la notificaci
     * */
    private void createNotification(){
    	sendMessagetoService(TWService.MSG_CREATE_NOTIFICATION, null);
    }
}
