package trailwalker.track.ui.data;

/**
 * Classe utilitzada per altres classes per transportar informaci referent a configuraci de l'aplicaci (temps entre captures, URL API REST...)
 * @author Albert Jord Vinardell
 * @version Desembre 2012
 * */

public class ApplicationSettingsInformation {
	
    //*******************************************************************************
    //Variables
    //*******************************************************************************
	private long time_between_location_updates = 0;
	private long distance_between_location_updates = 0;
	private String restUrl = "";
	private boolean useNetworkLocation = false;
	private boolean useGPSLocation = false;
	private boolean useGPSFirst = false;
	
    //*******************************************************************************
    //Constructor
    //*******************************************************************************
	/**
	 * Constructor
	 * @param time_between_location_updates: long que cont el temps (milisegons) entre captures de posici
	 * @param distance_between_location_updates: long que cont la distncia (metres) entre captures de posici
	 * @param restUrl String que cont la URL de la API REST a on s'envien les posicions
	 * @param useNetworkLocation boole que indica si l'aplicaci ha d'utilitzar posicionament per xarxes mbils
	 * @param useGPSLocation boole que indica si l'aplicaci ha d'utilitzar posicioanment per GPS
	 * @param useGPSFirst boole que indica si, si s'utilitzen Xarxes mbils i GPS, que predomini l's del GPS per sobre del de xarxa
	 * */
	public ApplicationSettingsInformation(long time_between_location_updates,
			long distance_between_location_updates, String restUrl,
			boolean useNetworkLocation, boolean useGPSLocation, boolean useGPSFirst) {
		super();
		this.time_between_location_updates = time_between_location_updates;
		this.distance_between_location_updates = distance_between_location_updates;
		this.restUrl = restUrl;
		this.useNetworkLocation = useNetworkLocation;
		this.useGPSLocation = useGPSLocation;
		this.useGPSFirst = useGPSFirst;
	}
	
    //*******************************************************************************
    //Getters i Setters
    //*******************************************************************************
	/**
	 * Torna el temps (milisegons) entre captures de posici
	 * @return long que cont el temps (milisegons) entre captures de posici
	 * */
	public long getTime_between_location_updates() {
		return time_between_location_updates;
	}
	
	/**
	 * Funci per fixar el temps (milisegons) entre captures de posici
	 * @param time_between_location_updates long que cont el temps (milisegons) entre captures de posici
	 * */
	public void setTime_between_location_updates(long time_between_location_updates) {
		this.time_between_location_updates = time_between_location_updates;
	}
	
	/**
	 * Torna el distncia (metres) entre captures de posici
	 * @return long que cont la distncia (metres) entre captures de posici
	 * */
	public long getDistance_between_location_updates() {
		return distance_between_location_updates;
	}

	/**
	 * Funci per fixar la distncia (metres) entre captures de posici
	 * @param distance_between_location_updates long que cont la distncia (metres) entre captures de posici
	 * */
	public void setDistance_between_location_updates(
			long distance_between_location_updates) {
		this.distance_between_location_updates = distance_between_location_updates;
	}
	
	
	/**
	 * Torna la URL de la API REST a on s'envien les posicions
	 * @return {@link String} que cont la URL de la API REST a on s'envien les posicions
	 * */
	public String getRestUrl() {
		return restUrl;
	}
	
	
	/**
	 * Fixa la URL de la API REST a on s'envien les posicions 
	 * @param restUrl {@link String} que cont la URL de la API REST
	 */
	public void setRestUrl(String restUrl) {
		this.restUrl = restUrl;
	}
	
	
	/**
	 * Funci que torna un boole en funci de si la informaci continguda descriu que s'ha d'utilitzar posicionament per Xarxes mbils
	 * @return {@link Boolean} que indica si s'ha d'utilitzar posicionament per Xarxes mbils
	 */
	public boolean isUseNetworkLocation() {
		return useNetworkLocation;
	}

	/**
	 * Fixa si s'ha d'utilitzar posicionament per Xarxes mbils
	 * @param useNetworkLocation boole per indicar si s'ha d'utilitzar posicionament per Xarxes mbils
	 */
	public void setUseNetworkLocation(boolean useNetworkLocation) {
		this.useNetworkLocation = useNetworkLocation;
	}

	/**
	 * Funci que torna un boole en funci de si la informaci continguda descriu que s'ha d'utilitzar posicionament per GPS
	 * @return
	 */
	public boolean isUseGPSLocation() {
		return useGPSLocation;
	}

	/**
	 * Fixa si s'ha d'utilitzar posicionament per GPS
	 * @param useGPSLocation boole per indicar si s'ha d'utilitzar posicionament per GPS
	 */
	public void setUseGPSLocation(boolean useGPSLocation) {
		this.useGPSLocation = useGPSLocation;
	}

	/**
	 * Funci que torna un boole en funci de si la informaci continguda descriu que s'ha d'utilitzar posicionament per GPS per sobre de xarxes mbils
	 * @return
	 */
	public boolean isUseGPSFirst() {
		return useGPSFirst;
	}

	/**
	 * Fixa si s'ha d'utilitzar posicionament per GPS per sobre de xarxes mbils
	 * @param useGPSLocation boole per indicar si s'ha d'utilitzar posicionament per GPS per sobre de xarxes mbilx
	 */
	public void setUseGPSFirst(boolean useGPSFirst) {
		this.useGPSFirst = useGPSFirst;
	}
	
}
